/* grammar.y - dataswap bison grammar */

%{
#include "lddl.h"	/* LDDL_INT8, LDDL_INT16, etc. */
#include "ds.h"
#include "misc.h"
#include "args.h"

static char *cvsid = "$Id: grammar.y,v 1.1.1.1 2000/03/24 20:10:24 marty Exp $";

#ifdef USE_INT_FOR_LONG
/* dec alpha long is 8 bytes, but int is 4 */
#define long int
#define u_long int
#endif

static valstruct V;
static int Curtype;
static int Cur_group_id;
static char *Cur_group_name;
static int Defining_group = 0;	/* if nonzero, defining a "group" */
static int In_sig = 0;		/* if nonzero, processing a "signature" */

#define YYDEBUG 1
int yydebug = 0;
%}

%union {
char * str;	/* for STRING */
double fval;	/* for floats/doubles */
int ival;	/* for everything else */
}

%token <ival>	AND
%token <ival>	ARROW
%token <ival>	ENDIAN		/* endian definition */
%token <ival>	EVT		/* evt definition */
%token <fval>	FLOATNUM	/* floating pt value */
%token <ival>	GROUP
%token <ival>	GROUPEND
%token <ival>	MASK
%token <ival>	NUM		/* integer value */
%token <ival>	OFFSET
%token <ival>	OPTIONAL
%token <ival>	PRINT_EVTS
%token <ival>	SIG		/* evt signature */
%token <str>	STRING		/* character string */
%token <ival>	SEPARATOR
%token <ival>	T_BYTE
%token <ival>	T_DOUBLE
%token <ival>	T_FLOAT
%token <ival>	T_SHORT
%token <ival>	T_LONG
%token <ival>	TYPE
%token <ival>	VALUE

%type <ival>	arg
%type <ival>	arglist
%type <ival>	args
%type <str>	assign_from
%type <str>	assign_to
%type <ival>	cond
%type <ival>	cond_args
%type <ival>	count
%type <ival>	endian
%type <ival>	evt
%type <ival>	line
%type <ival>	mask
%type <ival>	number
%type <ival>	offset
%type <ival>	signature
%type <ival>	signature_start
%type <ival>	group_def_end
%type <ival>	group_def
%type <ival>	typespec
%type <ival>	type

%%
input :	  /* empty */
	| input line
	| input '\n'
;

line :	  typespec
	| group_def
	| group_def_end
	| evt
	| signature
	| endian
	| PRINT_EVTS	{ print_evts(); }
;

group_def :	  GROUP STRING	{
		    if (Defining_group) {
			fprintf(stderr,
			    "group def still in prior group near line %d\n",
			    lineno_get());
			return 1;
		    }
		    group_init($2);
		    Defining_group = 1;
		  }
;

group_def_end :  GROUPEND {
		    if (Defining_group) {
			group_end();
			Defining_group = 0;
		    } else {
			fprintf(stderr, "unmatched 'group_end' near line %d\n",
			    lineno_get());
			return 1;
		    }
		}
;

typespec: type count args '\n'	{
	    if (Defining_group) {
		/* compiling a group or a group into a group */
		group_add($1, $2, NULL, NULL, 0, NULL, 0);
	    } else if ($1 == LDDL_GROUP) {
		/* insert group */
		newspec($1, $2, NULL, NULL, 0, NULL, 0, Cur_group_id,
		    Cur_group_name);
	    } else {
		newspec($1, $2, NULL, NULL, 0, NULL, 0, -1, NULL);
	    }
	  }
	| type count number mask args '\n'	{
	    if (Defining_group) {
		if ($1 != LDDL_GROUP) {
		    /* compiling a group */
		    group_add($1, $2, NULL, NULL, 0, &V, $4);
		} else {
		    /* compiling a group into a group */
		    fprintf(stderr,
			"value compiling group into group near line %d\n",
		    	lineno_get());
		    return 1;
		}
	    } else if ($1 == LDDL_GROUP) {
		/* insert group */
		fprintf(stderr,"group with value near line %d\n",
		    lineno_get());
		return 1;
	    } else {
		newspec($1, $2, NULL, NULL, 0, &V, $4, -1, NULL);
	    }
	}
	| type count number mask OPTIONAL args '\n' {
	    if (Defining_group) {
		if ($1 != LDDL_GROUP) {
		    /* compiling a group */
		    group_add($1, $2, NULL, NULL, 1, &V, $4);
		} else {
		    /* compiling a group into a group */
		    fprintf(stderr,
		    	"optional compiling group into group near line %d\n",
			lineno_get());
		    return 1;
		}
	    } else if ($1 == LDDL_GROUP) {
		/* insert group */
		fprintf(stderr,"group with value near line %d\n",
		    lineno_get());
		return 1;
	    } else {
		newspec($1, $2, NULL, NULL, 1, &V, $4, -1, NULL);
	    }
	}
	| type count assign_to args '\n'	{
	    if (Defining_group) {
		if ($1 != LDDL_GROUP) {
		    /* compiling a group */
		    group_add($1, $2, NULL, $3, 0, NULL, 0);
		} else {
		    /* compiling a group into a group */
		    fprintf(stderr,
		    	"to_var compiling group into group near line %d\n",
			lineno_get());
		    return 1;
		}
	    } else if ($1 == LDDL_GROUP) {
		/* insert group */
		fprintf(stderr,"group with to_var near line %d\n",
		    lineno_get());
		return 1;
	    } else {
		newspec($1, $2, NULL, $3, 0, NULL, 0, -1, NULL);
	    }
	}
	| type assign_from args '\n' {
	    if (Defining_group) {
		/* compiling a group or a group into a group */
		group_add($1, 0, $2, NULL, 0, NULL, 0);
	    } else if ($1 == LDDL_GROUP) {
		/* insert group into nodes */
		newspec($1, 0, $2, NULL, 0, NULL, 0, Cur_group_id,
		    Cur_group_name);
	    } else {
		newspec($1, 0, $2, NULL, 0, NULL, 0, -1, NULL);
	    }
	}
	| type OPTIONAL '\n' {
	    /* error */
	    fprintf(stderr, "optional with no value at line %d\n",
	    	lineno_get()-1);
	    return 1;
	}
;

type :	  T_SHORT	{ $$ = LDDL_INT16; Curtype = $$; }
	| T_LONG	{ $$ = LDDL_INT32; Curtype = $$; }
	| T_BYTE	{ $$ = LDDL_INT8; Curtype = $$; }
	| T_FLOAT	{ $$ = LDDL_FLOAT32; Curtype = $$; }
	| T_DOUBLE	{ $$ = LDDL_FLOAT64; Curtype = $$; }
	| STRING	{
	    /* insert group definition */
	    int ret;
	    ret = group_defined($1);
	    if (ret == -1) {
		fprintf(stderr, "No such group '%s' near line %d\n",
		    $1, lineno_get());
		return 1;
	    }
	    Cur_group_name = str_alloc(Cur_group_name, $1);
	    strcpy(Cur_group_name, $1);
	    Cur_group_id = ret;
	    $$ = LDDL_GROUP;
	}
;

count :	  /* empty */		{ $$ = 1; }	/* default val = 1 */
	| NUM 
;

number:	  '=' NUM	{
		if (Curtype == LDDL_INT16) {
		    V.u.s = (unsigned short)$2;
		} else if (Curtype == LDDL_INT32) {
		    V.u.l = (unsigned long)$2;
		} else if (Curtype == LDDL_INT8) {
		    V.u.b = (unsigned char)$2;
		} else {
		    fprintf(stderr,
			"set non-integer type to integer number near line %d\n",
			lineno_get());
		    return 1;
		}
		V.is_set = 1;
	    }

	| '=' FLOATNUM	{
		if (Curtype == LDDL_FLOAT32) {
		    V.u.f = (float)$2;
		} else if (Curtype == LDDL_FLOAT64) {
		    V.u.d = (double)$2;
		} else {
		    fprintf(stderr,
			"set non-float type to float number near line %d\n",
			lineno_get());
		    return 1;
		}
		V.is_set = 1;
	    }
;

assign_to:	ARROW STRING	{ 
	$$ = $2;
    }
;

assign_from:	STRING	{
	$$ = $1;
    }
;

endian:		ENDIAN cond

signature:		signature_start cond	{ In_sig = 0; }
signature_start:	SIG STRING	{
			    In_sig = 1;
			    evt_kind_init($2);
    			}

cond:	  cond_args
	| cond_args AND cond
;

cond_args:  offset TYPE type '=' NUM mask {
		if (In_sig) {
		    evt_kind_value($1, $3, $5, $6);
		} else {
		    endian_set($1, $3, $5, $6);
		}
	  }
;

offset:	  /* empty */	{ $$ = 0; }
	| OFFSET NUM	{ $$ = $2; }
;

mask:	  /* empty */	{ $$ = 0; } /* mask of 0 means "don't mask" */
	| MASK NUM	{ $$ = $2; }
;

evt:	EVT STRING args {
	    evt_def($2);
	}
;

args:	  /* empty */	{ $$ = 0; }
	| SEPARATOR arglist
;

arglist:  arg
	| arglist arg
;

arg:	  STRING {
	    args_add($1);
	  }
	| NUM	 {
	    char str[128];
	    snprintf(str, 128, "%d", $1);
	    args_add(str);
	  }
	| FLOATNUM {
	    char str[128];
	    snprintf(str, 128, "%f", $1);
	    args_add(str);
	  }
;
%%

/* mitten_drin ("in the middle of") - this routine can be called after
 * yyparse() has returned to see if we were in the middle of something when
 * we ran out of input. The function prototype is found in dataswap.h. */
int
mitten_drin(void)
{
    if (Defining_group) {
	fprintf(stderr, "End of input in the middle of group definition.\n");
	return 1;
    }
    return 0;
}
