/* data.c - experiment specific code for each experiment goes here. As a
 * start, put your data analysis code in the function 'process_evt' below.
 *
 * You will also probably have to modify the MAXDATA macro below.
 *
 * Marty Olevitch, Mar 01
 */

#include <stdio.h>	/* fprintf */

#include "data.h"

static char *cvsid = "$Id: data.c,v 1.1 2001/03/07 20:46:22 marty Exp $";

/* MAXDATA - max number of bytes in any event. */
static const long MAXDATA = 260000L;

static char *Data;	/* ptr to start of event's accumulated data */
static unsigned long Evtno = 0L;

enum {
    TYPE_REAL = 0,
    TYPE_LEDCAL = 2,
    TYPE_PEDCAL = 3,
    TYPE_REPLY = 4,
    TYPE_PHAONLY = 9,
    NSCAL = 15,		/* no. of scalars */
    NWUPHA = 56,	/* no. of Wash U phas */
    NMARSHALLPHA = 64,	/* no. of Marshall phas */
};

/* offsets and length in units of 16-bit words */
enum {
    OFF_TYPE = 2,
    LEN_STD = 18,
    OFF_PHA = LEN_STD + 2,
    LEN_PHA = 122,
    OFF_DISC = LEN_STD + LEN_PHA + 2,
    LEN_DISC = 52,
    OFF_VID = LEN_STD + LEN_PHA + LEN_DISC + 3,
    LEN_VID = 127676,
};

struct stdstuff {
    unsigned short type;
    unsigned short exp_id;
    unsigned long size;
    unsigned long evtno;
    unsigned long abs_evtno;
    unsigned long clockticks;
    unsigned long vidsize1;
    unsigned long vidsize2;
};

struct discrimstuff {
    unsigned char cam_discrim;
    unsigned char iigain;
    unsigned short status;
    unsigned short selscalar[NSCAL];
    unsigned short sumscalar[NSCAL];
};

struct phastuff {
    unsigned short wupha[NWUPHA];
    unsigned short marshallpha[NMARSHALLPHA];
};

struct realevt {
    struct stdstuff std;
    struct phastuff pha;
    struct discrimstuff disc;
    unsigned short vid[LEN_VID];
};

static void fill_evt_struct(struct realevt *r);

void
set_data(char *d)
{
    Data = d;
}

long
get_maxdata(void)
{
    return MAXDATA;
}

void
process_evt(void)
{
    unsigned char *cp;
    unsigned long *lp;
    unsigned short *sp;
    struct realevt r;

    Evtno++;

    /* type */
    sp = (unsigned short *)Data;
    sp += 2;
    if (*sp != TYPE_REAL) {
	/* Just mess with real events. */
	return;
    }

    fill_evt_struct(&r);

    /* Process evt here -- your code goes here! */
    fprintf(stderr, "%lu\n", r.std.abs_evtno);
}

static void
fill_evt_struct(struct realevt *r)
{
    unsigned char *cp;
    int i;
    unsigned long *lp;
    unsigned short *sp;

    sp = (unsigned short *)Data;
    sp += OFF_TYPE;

    r->std.type = *sp++;
    r->std.exp_id = *sp++;

    lp = (unsigned long *)sp;
    r->std.size = *lp++;
    r->std.evtno = *lp++;
    r->std.abs_evtno = *lp++;
    r->std.clockticks = *lp++;
    r->std.vidsize2 = *lp++;
    lp++;
    r->std.vidsize1 = *lp;

    sp = (unsigned short *)Data;
    sp += OFF_PHA;	/* start of pha stuff */

    for (i=0; i < NWUPHA; i++) {
	r->pha.wupha[i] = *sp++;
    }
    for (i=0; i < NMARSHALLPHA; i++) {
	r->pha.marshallpha[i] = *sp++;
    }

    sp = (unsigned short *)Data;
    sp += OFF_DISC;

    cp = (unsigned char *)sp;
    r->disc.cam_discrim = *cp++;
    r->disc.iigain = *cp++;

    sp = (unsigned short *)cp;
    r->disc.status = *sp++;
    for (i=0; i < NSCAL; i++) {
	r->disc.selscalar[i] = *sp++;
    }
    for (i=0; i < NSCAL; i++) {
	r->disc.sumscalar[i] = *sp++;
    }

    sp = (unsigned short *)Data;
    sp += OFF_VID;
    memcpy(r->vid, sp, LEN_VID * sizeof(short));
}
