/* proc.c - generic data analysis framework using lddl library
 *
 * NOTE: put your data analysis code in the function 'process_evt' in the
 * file data.c. You shouldn't have to change this file.
 *
 * Marty Olevitch, Jan 01
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include <lddl.h>
#include "data.h"

static char *cvsid = "$Id: proc.c,v 1.3 2001/03/07 20:46:23 marty Exp $";

#ifndef STREQ
#define STREQ(a, b)	(strcmp((a), (b)) == 0)
#endif

static int evt(char *name, int argc, char **argv);
static int group(int argc, char **argv, char *grpname, char *evtname);
static int node(char *inp, char *swapp, int type, int count, int nbytes, int
    argc, char **argv, int native, char *group_name, char *evt_name);
static int post_evt(char *name, int argc, char **argv);

static void usage(void);

static char *Curp;	/* ptr to current location in Data buffer */
static char *Data;	/* ptr to start of event's accumulated data */
static unsigned long Evtbytes = 0L;
static unsigned long Evtcnt = 0L;
static int Evt_overflow;	/* nonzero if too much data in evt */
static long Maxdata;		/* max no. of bytes in any event */
static char *Progname;

int
main(int argc, char *argv[])
{
    int c;
    int errflg = 0;
    char *fmtfile = NULL;
    extern int optind;
    extern char *optarg;

    Progname = argv[0];

    while ((c = getopt(argc, argv, "f:Nn")) != EOF) {
	switch (c) {
	    case 'f':
	    	fmtfile = optarg;
		break;
	    case 'N':
	    	/* assume native endianness */
		lddl_set_native(1);
		break;
	    case 'n':
	    	/* assume non-native endianness */
		lddl_set_native(0);
		break;
	    case '?':
	    default:
	    	errflg++;
		break;
	}
    }

    if (errflg) {
	usage();
	exit(1);
    }

    /* Note: the get_maxdata() routine is found in the file 'data.c' */
    Maxdata = get_maxdata();
    Data = (char *)malloc(Maxdata);
    if (Data == NULL) {
	fprintf(stderr, "%s: can't malloc (%s). Exiting.\n",
	    Progname, strerror(errno));
	exit(1);
    }
    set_data(Data);

    lddl_set_group_func(group);
    lddl_set_evt_func(evt);
    lddl_set_post_evt_func(post_evt);
    lddl_set_node_func(node);
    (void)lddl_search_for_evt_start(1);
    lddl_start(fmtfile, argc-optind, argv+optind, Progname);
    /* printf("%d\n", Evtcnt); */
    return 0;
}

/* evt - start accumulating the data of a new event */
static int 
evt(char *name, int argc, char **argv)
{
    Curp = Data;
    Evtbytes = 0L;
    Evt_overflow = 0;
    return 0;
}

/* post_evt - process the accumulated data of the event */
static int 
post_evt(char *name, int argc, char **argv)
{
    if (Evt_overflow) {
	fprintf(stderr, "%s: event overflow\n", Progname);
	return 0;
    }
    process_evt();
    return 0;
}

/* group - no-op for now */
static int
group(int argc, char **argv, char *group_name, char *evtname)
{
    return 0;
}

/* node - just copy the data */
static int
node(char *inp, char *swapp, int type, int count, int nbytes, int
    argc, char **argv, int native, char *group_name, char *evt_name)
{
    char *p;

    if (Evtbytes + nbytes > Maxdata) {
	Evt_overflow = 1;
    }
    if (Evt_overflow) {
	return 0;
    }
    p = native ? inp : swapp;
    memcpy(Curp, p, nbytes);
    Curp += nbytes;
    Evtbytes += nbytes;
    return 0;
}

static void
usage(void)
{
    fprintf(stderr,
    	"\nusage: %s -f fmtfile [-N] [-n] [-o outfile] [file ...]\n",
    	Progname);
    fprintf(stderr, "     -f fmtfile    name of format file\n");
    fprintf(stderr, "     -N            data IS in native endianness\n");
    fprintf(stderr, "     -n            data IS NOT in native endianness\n");
    fprintf(stderr, "     file          input data file(s) (def. stdin)\n");
}
