
/*
 Copyright (C) 2003 Martin A. Olevitch
 Written by Martin A. Olevitch (marty@cosray.wustl.edu)

 This file is part of LDDL.

 LDDL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 LDDL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with LDDL, see the file COPYING; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* args.c - argc, argv passed to applications */

#include <stdio.h>	/* NULL */
#include <string.h>

#include "args.h"
#include "misc.h"

static char *cvsid = "$Id: args.c,v 1.1.1.1 2000/03/24 20:10:24 marty Exp $";

/* Cur_argc - arg count for current node. Incremented by args_add()
 * and set to 0 by fill_node(). */
static int Cur_argc = 0;

#define MAXARGS 1024

/* Cur_argv - arg vector for current node. Allocated/incremented by
 * args_add() and set to NULL by fill_node(). */
static char *Cur_argv[MAXARGS];

void
args_reset_cur_argc(void)
{
    Cur_argc = 0;
}

int 
args_cur_argc(void)
{
    return Cur_argc;
}

char **
args_cur_argv(void)
{
    char **p;
    int i;

    if (Cur_argc <= 0) {
	return (char **)NULL;
    }

    p = (char **)malloc(sizeof(char *) * Cur_argc);
    if (p == NULL) {
	fatal("args_cur_argv: can't malloc p\n");
    }
    
    for (i=0; i < Cur_argc; i++) {
	p[i] = str_alloc(NULL, Cur_argv[i]);
	strcpy(p[i], Cur_argv[i]);
    }

    return p;
}

void
args_add(char *name)
{
    Cur_argv[Cur_argc] = str_alloc(NULL, name);
    strcpy(Cur_argv[Cur_argc], name);
    Cur_argc++;
}

