
/*
 Copyright (C) 2003 Martin A. Olevitch
 Written by Martin A. Olevitch (marty@cosray.wustl.edu)

 This file is part of LDDL.

 LDDL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 LDDL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with LDDL, see the file COPYING; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* dataswap.c
 *
 * Marty Olevitch, Feb 2000
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include "lddl.h"

static char *cvsid = "$Id: dataswap.c,v 1.1.1.1 2000/03/24 20:10:24 marty Exp $";

int node(char *inp, char *swapp, int type, int count, int nbytes, int
    argc, char **argv, int native, char *group_name, char *evt_name);

static char *Progname;
static void usage(void);
static FILE *Ofp;
static char *Outfilename;

int
main(int argc, char *argv[])
{
    int c;
    int errflg = 0;
    char *fmtfile = NULL;
    extern int optind;
    extern char *optarg;

    Progname = argv[0];

    Ofp = stdout;
    Outfilename = strdup("stdout");
    if (Outfilename == NULL) {
	fprintf(stderr, "%s: can't malloc!\n", Progname);
	exit(1);
    }

    while ((c = getopt(argc, argv, "f:Nno:")) != EOF) {
	switch (c) {
	    case 'f':
	    	fmtfile = optarg;
		break;
	    case 'o':
	    	Outfilename = optarg;
		Ofp = fopen(Outfilename, "w");
		if (Ofp == NULL) {
		    fprintf(stderr, "%s: can't open output file '%s' (%s)\n",
		    	Progname, Outfilename, strerror(errno));
		    exit(1);
		}
		break;
	    case 'N':
	    	/* assume native endianness */
		lddl_set_native(1);
		break;
	    case 'n':
	    	/* assume non-native endianness */
		lddl_set_native(0);
		break;
	    case '?':
	    default:
	    	errflg++;
		break;
	}
    }

    if (errflg) {
	usage();
	exit(1);
    }

    lddl_set_node_func(node);
    lddl_return_modified_value(0);
    lddl_return_duplicate_node(0);
    lddl_start(fmtfile, argc-optind, argv+optind, Progname);
    return 0;
}

int
node(char *inp, char *swapp, int type, int count, int nbytes, int
    argc, char **argv, int native, char *group_name, char *evt_name)
{
    size_t n;
    n = fwrite(swapp, sizeof(char), nbytes, Ofp);
    if (n == 0) {
	if (ferror(stdout)) {
	    fprintf(stderr, "%s: write error on file '%s' (%s)\n",
	    	Progname, Outfilename, strerror(errno));
	    exit(1);
	}
    }
}

static void
usage(void)
{
    fprintf(stderr,
    	"\nusage: %s -f fmtfile [-N] [-n] [-o outfile] [file ...]\n",
    	Progname);
    fprintf(stderr, "     -f fmtfile    name of format file\n");
    fprintf(stderr, "     -N            data IS in native endianness\n");
    fprintf(stderr, "     -n            data IS NOT in native endianness\n");
    fprintf(stderr, "     -o outfile    output file (default stdout)\n");
    fprintf(stderr, "     file          input data file(s) (def. stdin)\n");
}
