
/*
 Copyright (C) 2003 Martin A. Olevitch
 Written by Martin A. Olevitch (marty@cosray.wustl.edu)

 This file is part of LDDL.

 LDDL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 LDDL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with LDDL, see the file COPYING; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* list.c
 *
 * singly-linked insert-only lists
 *
 * Marty Olevitch, Feb 2000
 */

#include <stdio.h>	/* NULL on some systems */

#include "list.h"
#include "misc.h"

static char *cvsid = "$Id: list.c,v 1.1.1.1 2000/03/24 20:10:24 marty Exp $";

#define BAD_ID(i)	((id) < 0 || (id) >= N)

struct listnode {
    void *p;			/* ptr to users data item */
    struct listnode *next;	/* ptr to next node */
};
typedef struct listnode lnode;

struct list {
    lnode *cur;		/* ptr to current node for list traversal */
    lnode *head;	/* ptr to first node of list (no data) */
    lnode *tail;	/* ptr to last node of list (may have data) */
};
typedef struct list list;

#define NLIST 1024
static list List[NLIST];

static int N = 0;	/* current number of lists */

/* list_new - initialize a new list */
int
list_new(void)
{
    lnode *lp;

    if (N >= NLIST) {
	return -1;
    }

    lp = emalloc(sizeof(lnode));
    lp->next = NULL;

    List[N].head = lp;
    List[N].tail = lp;
    List[N].cur = NULL;

    N++;
    return N-1;
}

/* list_add - add a node to the end of the list */
int
list_add(int id, void *p)
{
    lnode *lp;

    if (BAD_ID(id)) {
	return -1;
    }

    lp = emalloc(sizeof(lnode));

    lp->p = p;
    lp->next = NULL;
    List[id].tail->next = lp;
    List[id].tail = lp;

    return 0;
}

/* list_first - move the list's cur ptr to the first data node */
int
list_first(int id)
{
    if (BAD_ID(id)) {
	return -1;
    }
    List[id].cur = List[id].head->next;
    return 0;
}

/* list_val - retrieve the data pointed to in the list's cur node and
 * advance the cur ptr. Returns -1 on bad list id, 0 at end of list (data
 * is not usable), or 1 if all went well (data is usable). */
int
list_val(int id, void **p)
{
    if (BAD_ID(id)) {
	return -1;
    }
    if (List[id].cur == NULL) {
	return 0;
    }
    *p = List[id].cur->p;
    List[id].cur = List[id].cur->next;
    return 1;
}
