
/*
 Copyright (C) 2003 Martin A. Olevitch
 Written by Martin A. Olevitch (marty@cosray.wustl.edu)

 This file is part of LDDL.

 LDDL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 LDDL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with LDDL, see the file COPYING; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
 * snprintf() quicky using temporary files.
 * Not stress tested, but it seems to work.
 *
 * Returns the number of bytes that would have been output by printf.
 * Does not check whether a negative value if passed in for n (as it's unsigned);
 * some implementations cast n to int and than compare with 0.
 *
 * Casper Dik (Casper.Dik@Holland.Sun.COM)
 */

#ifndef HAVE_SNPRINTF

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>

static char *cvsid = "$Id: snprintf.c,v 1.2 2001/01/03 19:49:27 marty Exp $";

#ifndef SEEK_SET
#define SEEK_SET 0
#endif

int snprintf(char * buf, size_t n, const char *fmt, ...)
{
    va_list ap;
    int ret;

    va_start(ap, fmt);
    ret = vsnprintf(buf, n, fmt, ap);
    va_end(ap);
    return ret;
}

static char template[] = "/tmp/snprintfXXXXXX";

int vsnprintf(char *buf, size_t n, const char *fmt, va_list ap)
{
    char templ[sizeof(template)];
    int s;
    int ret, cnt = 0, nread;
    FILE *fp;

    strcpy(templ,template);
    s = mkstemp(templ);
    if (s < 0) {
	return -1;
    }
    (void) unlink(templ);

    fp = fdopen(s, "w+");
    if (fp == NULL) {
	close(s);
	return -1;
    }

    ret = vfprintf(fp, fmt, ap);
    if (ret < 0 || fseek(fp, 0, SEEK_SET) < 0) {
	fclose(fp);
	return -1;
    }

    while (cnt < n && (nread = fread(buf + cnt, 1, n - cnt, fp)) > 0) {
	cnt += nread;
    }
    /*buf[cnt-1] = '\0'; /* cnt is at most n */
    buf[cnt] = '\0'; /* cnt is at most n */

    fclose(fp);
    return ret;
}
#endif /* HAVE_SNPRINTF */
