
/*
 Copyright (C) 2003 Martin A. Olevitch
 Written by Martin A. Olevitch (marty@cosray.wustl.edu)

 This file is part of LDDL.

 LDDL is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 LDDL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with LDDL, see the file COPYING; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* var.c - implements "internal" variables */

#include <stdio.h>

#include "list.h"
#include "misc.h"
#include "var.h"
#include <string.h>

static char *cvsid = "$Id: var.c,v 1.1.1.1 2000/03/24 20:10:24 marty Exp $";

/* varnode - stores the name and value of our "internal" variables */
struct varnode {
    char *name;
    int value;
};
typedef struct varnode varnode;

static int Var_list_id;		/* varnode list id */

int
var_init(void)
{
    Var_list_id = list_new();
    if (Var_list_id < 0) {
	return -1;
    }
    return 0;
}

/* var_getval - return the value of the internal variable with 'name' in
 * the int pointed to by 'value'. */
void
var_getval(char *name, int *value)
{
    varnode *vp;
    if (name == NULL || value == NULL) {
	fatal("var_getval: null name or null value\n");
    }
    if (list_first(Var_list_id)) {
	fatal("var_getval: bad list_first\n");
    }
    while (list_val(Var_list_id, (void **)&vp) > 0) {
	if (STREQ(name, vp->name)) {
	    *value = vp->value;
	    return;
	}
    }
    fatal("var_getval: can't find variable '%s'\n", name);
}

void
var_setval(char *name, int value)
{
    varnode *vp;
    if (name == NULL) {
	fatal("var_setval: null name\n");
    }
    if (list_first(Var_list_id) == 0) {
	/* something on the list */
	while (list_val(Var_list_id, (void **)&vp) > 0) {
	    if (STREQ(vp->name, name)) {
		vp->value = value;
		return;
	    }
	}
    }

    vp = emalloc(sizeof(varnode));
    vp->name = str_alloc(NULL, name);
    strcpy(vp->name, name);
    vp->value = value;
    if (list_add(Var_list_id, vp)) {
	fatal("var_setval: bad list_add\n");
    }
}

